(* runtime config, solyga@gmx.de, 2007..2024-12-15 *)
IMPLEMENTATION MODULE Config;

IMPORT Lib, Str, IO;
IMPORT Args, LC;

(*# save, call( o_a_copy => off ) *)

PROCEDURE ShowVerStr();
BEGIN
  IO.WrStr( prg ); IO.WrStr( ' v' ); IO.WrCard( ver, 1 ); IO.WrStr( ' (' ); IO.WrStr( ext ); IO.WrStr( '): Raw Disk Access.' );
END ShowVerStr;

PROCEDURE ShowVer();
BEGIN
  ShowVerStr();
  IO.WrLn();
END ShowVer;

PROCEDURE ParseArgs();

  PROCEDURE Help();
  BEGIN
    IO.WrStr( '(* ' );
    ShowVerStr();
    IO.WrStr( ' *)' );
    IO.WrLn();
    IO.WrStr( 'author = "solyga@gmx.de" .' );
    IO.WrLn();
    IO.WrStr( 'syntax = "' ); IO.WrStr( prg ); IO.WrStr( '" { option } drive [ action ] .' );
    IO.WrLn();
    IO.WrStr( 'option = "-h"      (* show this help and terminate *)' );
    IO.WrLn();
    IO.WrStr( '       | "-v"      (* increase verbosity *)' );
    IO.WrLn();
    IO.WrStr( '       | "-b" hex  (* set address of first sector *)' );
    IO.WrLn();
    (*
    IO.WrStr( '       | "-e" hex  (.* set (succsessor of) last sector *.)' );
    IO.WrLn();
    *)
    IO.WrStr( '       | "-n" hex  (* set sector count *)' );
    IO.WrLn();
    IO.WrStr( '       | "-x" hex  (* set value to write *) .' );
    IO.WrLn();
    IO.WrStr( 'drive  = num       (* set drive number (0..255) *) .' );
    IO.WrLn();
    IO.WrStr( 'action = "read"    (* read sectors *)' );
    IO.WrLn();
    IO.WrStr( '       | "write"   (* write value to sectors *)' );
    IO.WrLn();
    IO.WrStr( '       | "sign"    (* write sector addresses to sectors *) .' );
    IO.WrLn();
  END Help;

  PROCEDURE Error( s: ARRAY OF CHAR );
  BEGIN
    IO.WrStr( 'ERROR: ' );
    IO.WrStr( s );
    IO.WrStr( ': "' );
    IO.WrStr( Args.str );
    IO.WrChar( '"' );
    IO.WrLn();
    IO.WrStr( 'Try "' );
    IO.WrStr( prg );
    IO.WrStr( ' -?" for help.' );
    IO.WrLn();
    HALT;
  END Error;

  PROCEDURE Value();
  BEGIN
    IF ~ LC.MkVal( data.val, Args.str ) THEN Error( 'Invalid value' ) END;
    data.wid := Args.len DIV 2 + ORD( Args.len MOD 2 > 0 );
    Args.Read();
  END Value;

  PROCEDURE Number( VAR x: ARRAY OF BYTE );
  BEGIN
    IF ~ LC.MkVal( x, Args.str ) THEN Error( 'Invalid sector number' ) END;
    Args.Read();
  END Number;

  PROCEDURE Options();
  BEGIN
    WHILE Args.sym = Args.symOpt DO
      IF Str.Compare( Args.str, '?' ) = 0 THEN
        Help();
        HALT;
      ELSIF Str.Compare( Args.str, 'h' ) = 0 THEN
        Help();
        HALT;
      ELSIF Str.Compare( Args.str, 'v' ) = 0 THEN
        LC.Inc( data.vrb );
        Args.Read();
      ELSIF Str.Compare( Args.str, 'x' ) = 0 THEN
        Args.Read();
        Value();
      ELSIF Str.Compare( Args.str, 'b' ) = 0 THEN
        Args.Read();
        Number( data.beg );
      ELSIF Str.Compare( Args.str, 'n' ) = 0 THEN
        Args.Read();
        Number( data.cnt );
      ELSE
        Error( 'Unknown option' );
      END;
    END;
  END Options;

  PROCEDURE Drive();
  VAR
    x: LONGCARD;
    ok: BOOLEAN;
  BEGIN
    IF Args.sym # Args.symArg THEN Error( 'Expected drive number' ) END;
    x := Str.StrToCard( Args.str, 10, ok );
    IF ~ ok OR ( x > 255 ) THEN Error( 'Invalid drive number' ) END;
    data.drv := SHORTCARD( x );
    Args.Read();
  END Drive;

  PROCEDURE Common( s: ARRAY OF CHAR ): CARDINAL;
  VAR
    l, i: CARDINAL;
  BEGIN
    l := Str.Length( s );
    IF l < Args.len THEN l := Args.len END;
    i := 0;
    WHILE ( l > 0 ) & ( CAP(Args.str[i]) = CAP(s[i]) ) DO INC( i ) END;
    RETURN i;
  END Common;

  PROCEDURE Action();
  BEGIN
    IF Args.sym # Args.symArg THEN RETURN END;
    IF Common( 'read' ) > 0 THEN
      data.act := actRead;
    ELSIF Common( 'write' ) > 0 THEN
      data.act := actWrite;
    ELSIF Common( 'sign' ) > 0 THEN
      data.act := actSign;
    ELSE
      Error( 'Invalid action' );
    END;
    Args.Read();
  END Action;

BEGIN
  Args.Reset();
  Args.Read();
  Options();
  Drive();
  Options();
  Action();
  Options();
  IF Args.sym # Args.symEOA THEN Error( 'Unexpected argument' ) END;
END ParseArgs;

PROCEDURE Show();
VAR
  s: TValStr;
BEGIN
  s := '';
  IO.WrStr( 'Config.data:' );
  IO.WrLn();
  IO.WrStr( ' vrb ' ); IO.WrShtCard( data.vrb, 1 );
  IO.WrLn();
  IO.WrStr( ' drv ' ); IO.WrShtCard( data.drv, 1 );
  IO.WrLn();
  IO.WrStr( ' act ' ); IO.WrCard( ORD(data.act), 1 );
  IO.WrLn();
  IO.WrStr( ' beg ' ); IF LC.MkStr( s, data.beg ) THEN IO.WrStr( s ) ELSE IO.WrStr( '?' ) END;
  IO.WrLn();
  IO.WrStr( ' cnt ' ); IF LC.MkStr( s, data.cnt ) THEN IO.WrStr( s ) ELSE IO.WrStr( '?' ) END;
  IO.WrLn();
  IO.WrStr( ' val ' ); IF LC.MkStrX( s, data.val, data.wid ) THEN IO.WrStr( s ) ELSE IO.WrStr( '?' ) END;
  IO.WrLn();
  IO.WrStr( ' wid ' ); IO.WrCard( data.wid, 1 );
  IO.WrLn();
END Show;

BEGIN
  Lib.Fill( ADR(data), SIZE(data), 0 );
  data.cnt := 1;
  data.wid := 4;

  ParseArgs();
  ShowVer();
  IF data.vrb > 1 THEN Show() END;
END Config.
