(* large cardinals *)
(* solyga@gmx.de, 2024-12-15 *)
IMPLEMENTATION MODULE LC;

IMPORT Lib, Str;

(*# save, call( o_a_copy => off ) *)

PROCEDURE Fill( VAR x: ARRAY OF BYTE; v: BYTE );
BEGIN
  Lib.Fill( ADR(x), SIZE(x), v );
END Fill;

PROCEDURE Digit( c: CHAR ): SHORTCARD;
BEGIN
  c := CAP( c );
  CASE c OF
  | '0'..'9': RETURN SHORTCARD( ORD( c ) - ORD( '0' ) );
  | 'A'..'Z': RETURN SHORTCARD( ORD( c ) - ORD( 'A' ) + 10 );
  ELSE
    RETURN MAX( SHORTCARD );
  END;
END Digit;

PROCEDURE MkVal( VAR v: ARRAY OF BYTE; s: ARRAY OF CHAR ): BOOLEAN;
VAR
  ok: BOOLEAN;
  l: CARDINAL;

  PROCEDURE Get( VAR x: SHORTCARD );
  BEGIN
    IF l > 0 THEN
      DEC( l );
      x := Digit( s[l] );
      IF x >= 10H THEN x := 0; ok := FALSE END;
    ELSE
      x := 0;
    END;
  END Get;

VAR
  i: CARDINAL;
  a, b: SHORTCARD;
BEGIN
  ok := TRUE;
  l := Str.Length( s );
  IF l = 0 THEN RETURN FALSE END;
  FOR i := 0 TO SIZE( v ) - 1 DO
    Get( a );
    Get( b );
    v[ i ] := BYTE( a + b * 10H );
  END;
  Get( a );
  WHILE ( a = 0 ) & ( l > 0 ) DO Get( a ) END;
  RETURN ok & ( l = 0 );
END MkVal;


PROCEDURE Char( x: SHORTCARD ): CHAR;
BEGIN
  IF x < 10 THEN
    RETURN CHR( x + ORD('0') );
  ELSE
    RETURN CHR( x - 10 + ORD('A') );
  END;
END Char;

PROCEDURE MkStrX( VAR s: ARRAY OF CHAR; v: ARRAY OF BYTE; n: CARDINAL ): BOOLEAN;
VAR
  ok: BOOLEAN;
  l: CARDINAL;

  PROCEDURE Put( x: SHORTCARD );
  BEGIN
    (*
    IF ( x = 0 ) & ( l = 0 ) THEN RETURN END;
    *)
    IF l >= SIZE( s ) THEN ok := FALSE; RETURN END;
    s[ l ] := Char( x );
    INC( l );
  END Put;

VAR
  i: CARDINAL;
  x: SHORTCARD;
BEGIN
  ok := TRUE;
  l := 0;
  IF n > SIZE( v ) THEN n := SIZE( v ) END;
  FOR i := n - 1 TO 0 BY -1 DO
    x := v[ i ];
    Put( x DIV 10H );
    Put( x MOD 10H );
  END;
  IF l = 0 THEN s[ 0 ] := '0'; l := 1 END;
  IF l < SIZE( s ) THEN s[ l ] := 0C END;
  RETURN ok;
END MkStrX;

PROCEDURE MkStr( VAR s: ARRAY OF CHAR; v: ARRAY OF BYTE ): BOOLEAN;
BEGIN
  RETURN MkStrX( s, v, SIZE(v) );
END MkStr;


PROCEDURE Len( x: ARRAY OF BYTE ): CARDINAL;
VAR
  l: CARDINAL;
BEGIN
  l := SIZE( x );
  WHILE ( l > 1 ) & ( x[l-1] = BYTE(0) ) DO DEC( l ) END;
  RETURN l;
END Len;

PROCEDURE Cmp0( x, y: BYTE ): INTEGER;
BEGIN
  IF SHORTCARD( x ) < SHORTCARD( y ) THEN RETURN -1 ELSIF SHORTCARD( x ) > SHORTCARD( y ) THEN RETURN 1 END;
  RETURN 0;
END Cmp0;

PROCEDURE Cmp( x, y: ARRAY OF BYTE ): INTEGER;
VAR
  lx, ly: CARDINAL;
  c: INTEGER;
BEGIN
  lx := Len( x ); ly := Len( y );
  IF lx < ly THEN RETURN -1 ELSIF lx > ly THEN RETURN 1 END;
  WHILE lx > 0 DO
    DEC( lx );
    c := Cmp0( x[lx], y[lx] );
    IF c # 0 THEN RETURN c END;
  END;
  RETURN 0;
END Cmp;

PROCEDURE Inc( VAR x: ARRAY OF BYTE );

  PROCEDURE DoInc( VAR x: BYTE ): BOOLEAN;
  BEGIN
    IF x = BYTE(255) THEN
      x := 0;
      RETURN TRUE;
    ELSE
      x := BYTE( SHORTCARD(x) + 1 );
      RETURN FALSE;
    END;
  END DoInc;

VAR
  c: BOOLEAN;
  i: CARDINAL;
BEGIN
  c := TRUE; i := 0;
  WHILE c & ( i < SIZE(x) ) DO
    c := DoInc( x[i] );
    INC( i );
  END;
  (* c = overflow *)
END Inc;

PROCEDURE Dec( VAR x: ARRAY OF BYTE );

  PROCEDURE DoDec( VAR x: BYTE ): BOOLEAN;
  BEGIN
    IF x = BYTE(0) THEN
      x := 255;
      RETURN TRUE;
    ELSE
      x := BYTE( SHORTCARD(x) - 1 );
      RETURN FALSE;
    END;
  END DoDec;

VAR
  c: BOOLEAN;
  i: CARDINAL;
BEGIN
  c := TRUE; i := 0;
  WHILE c & ( i < SIZE(x) ) DO
    c := DoDec( x[i] );
    INC( i );
  END;
  (* c = overflow *)
END Dec;

END LC.
